import module namespace rh = "http://freescale.com/revision-history" at "rhlib01.xq";

declare variable $today := xs:string( adjust-date-to-timezone( current-date(), () ) );

declare option saxon:default "'2000-01-01'";
declare variable $date-start as xs:string external;

declare option saxon:default "'1492-10-12'";
declare variable $date-end as xs:string external;

declare variable $filter_cols   := false();
declare variable $show_table    := false();
declare variable $show_filtered := false();

declare variable $earliest := xs:date( '1492-10-12' );

declare variable $rh-source-map := doc( './recipes/recipes.ditamap' );

declare variable $title :=  $rh-source-map//map/title;
declare variable $tempTableRows :=  local:build_temp_table_rows( $rh-source-map ) ;
(:declare variable $tempTableRows :=  local:build_temp_table_rows_13( $rh-source-map ) ;:)

(:  create set of table rows gleaned from all the  topic prologs of the map :)
declare function local:build_temp_table_rows_13 ( $map as node() ) {
  for $topicref in $map/*/topicref
    let $topic  := doc(resolve-uri(concat($topicref/@href," "), base-uri($rh-source-map)))
    let $person := $topic/topic/prolog/change-historylist/change-item[1]/change-person
      return    
        for $item in $topic/topic/prolog/change-historylist/change-item
          return 
            <row>{rh:add-s-atts( $item ) }
                <entry>{rh:add-s-atts( $item/change-completed ) }<p>{ data( $item/change-completed ) }</p></entry>    
                <entry>{data($topic/topic/title)}</entry>
                <entry>{ local:get-children( $item/change-person ) }</entry>    
                <entry>{ local:get-children( $item/change-summary ) }</entry>
            </row>
};

(:  create set of table rows gleaned from all the  topic prologs of the map :)
declare function local:build_temp_table_rows ( $map as node() ) {
  for $topicref in $map/*/topicref
    let $topic  := doc(resolve-uri($topicref/@href, base-uri($rh-source-map)))
    let $person := $topic/topic/prolog/change-historylist/change-item[1]/change-person
      return    
        for $item in $topic/topic/prolog/change-historylist/change-item
          return 
            <row>{rh:add-s-atts( $item ) }
                <entry>{rh:add-s-atts( $item/change-completed ) }<p>{ data( $item/change-completed ) }</p></entry>    
                <entry>{data($topic/topic/title)}</entry>
                <entry>{ local:get-children( $item/change-person ) }</entry>    
                <entry>{ local:get-children( $item/change-summary ) }</entry>
            </row>
};

(: return the items of a sequence (of nodes) as p elements, preserving select atts   :)
declare function local:get-children( $seq as node()* ) {
  for $item in $seq
    return
      <p>{rh:add-s-atts( $item ) }{ data( $item ) }</p>
};


declare function local:check-param2() {
  <topic id="history">
    <title>{ data( $title ) }</title>
    <body>
      <p audience="internal">Dates from { $date-start } to { $date-end }</p>
      <table><tgroup cols="2"><thead>{ rh:thead( $filter_cols ) }</thead><tbody>
          { 
            for $row-raw in $tempTableRows
              let $date-raw := $row-raw/entry[1]
              let $date := if ( $date-raw castable as xs:date ) then xs:date( $date-raw ) else $earliest
              let $row := rh:pick_cols( $row-raw, $filter_cols )
                return if ( $date != $earliest ) then 
                     ( if ( $date >= xs:date( $date-start ) and $date <= xs:date( $date-end ) )
                        then ( $row )
                        else ( if ( $show_filtered ) then rh:show_hidden_row( $row, "out of range" ) else () ) ) 
                    else rh:show_hidden_row( $row, "bad date" )
          }
      </tbody></tgroup></table>{ if ( $show_table ) then local:show_full_table ( $tempTableRows ) else () }
    </body>
  </topic>
};


declare function local:show_full_table ( $rows as node()* ) {
  <table audience="internal"><tgroup cols="4"><tbody>{ $rows }</tbody></tgroup></table>
};



(:  Here's the main query  :)

if ( not( $date-start castable as xs:date ) )
  then error((), "Invalid start parameter: must be date in form yyyy-mm-dd")
  else  if ( not( $date-end castable as xs:date ) )
    then error((), "Invalid end parameter: must be date in form yyyy-mm-dd")
    else local:check-param2()



(:
This XQuery gathers up release notes from DITA 1.3 topics and assembles them into a map.
yes:
  change-person
  change-completed
  change-summary
  [title]

no:
  change-organization
  change-revisionid
  change-request-reference
  change-started
:)


(:

<topic id="history">
  <title>{ data( $title ) }</title>
  <body><table><tgroup cols="2"><thead>{ local:thead() }</thead><tbody>
      { 
        for $row in $tempTableRows
          let $date-raw := $row/entry[1]
          let $date := xs:date( $date-raw )
            return 
              if ( $date >= $date-start and $date <= $date-end )
                then ( $row )
                else ( if ( $debugging ) then local:hide_row( $row ) else () )
      }
  </tbody></tgroup></table></body>
</topic>

:)